# Copyright © 2024 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
from contrast.api import TypeCheckedProperty
from contrast_vendor import structlog as logging

logger = logging.getLogger("contrast")

DATABASE_TYPE = "db"


class ArchitectureComponent:
    vendor = TypeCheckedProperty(str)
    url = TypeCheckedProperty(str)

    def __init__(self, vendor: str, db_name: str):
        self.vendor = vendor
        self.url = db_name or vendor or "default"

    def to_json(self):
        json = {
            "type": DATABASE_TYPE,
            "url": self.url,
        }

        # vendor must be a string that exactly matches a value from
        # Teamserver's flowmap/technologies.json > service > one of "name"
        if self.vendor:
            json["vendor"] = self.vendor

        return json



# Copyright © 2024 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
from collections.abc import Mapping
from typing import List, Optional, Protocol, Set

from contrast.agent.request import Request
from contrast.configuration.agent_config import AgentConfig
from contrast.reporting.teamserver_messages.base_ts_message import BaseTsMessage
from contrast_fireball import AssessFinding, DiscoveredRoute, ObservedRoute


class Reporter(Protocol):
    def initialize_application(self, config: AgentConfig, framework="") -> bool: ...

    def new_discovered_routes(self, routes: Set[DiscoveredRoute]): ...

    def new_observed_route(self, route: ObservedRoute): ...

    # new_findings is a batching method, but the Fireball client
    # will accept a single finding at a time and batch them internally.
    # When Fireball is the primary reporting client, we should consider
    # moving findings to a fire-and-forget model instead of batching.
    def new_findings(
        self, findings: List[AssessFinding], request: Optional[Request]
    ): ...

    # Legacy methods from direct reporting
    def add_message(self, msg: BaseTsMessage): ...

    def send_message(self, msg: BaseTsMessage): ...

    def retry_message(self, msg: BaseTsMessage): ...


def get_reporting_client(config: Mapping) -> Reporter:
    client_type = config.get("api.reporting_client")
    if client_type == "fireball":
        from contrast.reporting.fireball import Client
    elif client_type == "direct":
        from contrast.reporting.reporting_client import ReportingClient as Client
    else:
        raise ValueError(f"Invalid reporting client: {client_type}")

    return Client()



# Copyright © 2024 Contrast Security, Inc.
# See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
from functools import cached_property
from typing import List, Optional

from contrast.reporting.teamserver_responses.protect_rule import ProtectRule


class NGApplicationSettings:
    def __init__(self, settings_json=None):
        self._raw_settings = settings_json or {}

    @cached_property
    def disabled_assess_rules(self) -> List[str]:
        return (
            self._raw_settings.get("settings", {})
            .get("assessment", {})
            .get("disabledRules", [])
        )

    @cached_property
    def protection_rules(self) -> List[ProtectRule]:
        return [
            ProtectRule(r)
            for r in self._raw_settings.get("settings", {})
            .get("defend", {})
            .get("protectionRules", [])
        ]

    @cached_property
    def session_id(self) -> Optional[str]:
        return (
            self._raw_settings.get("settings", {})
            .get("assessment", {})
            .get("session_id", None)
        )

    @cached_property
    def sensitive_data_masking_policy(self) -> dict:
        return self._raw_settings.get("settings", {}).get(
            "sensitive_data_masking_policy", {}
        )

    # note: there are more fields on ApplicationSettings that we currently don't use

    def common_config(self):
        ts_exclusions = self._raw_settings.get("settings", {}).get("exceptions", {})
        return {
            "application.sensitive_data_masking_policy": self.sensitive_data_masking_policy,
            "application.url_exclusions": [
                {
                    "name": exclusion.get("name", ""),
                    "modes": exclusion.get("modes", []),
                    "urls": exclusion.get("urls", []),
                    "match_strategy": exclusion.get("matchStrategy", ""),
                    "protect_rules": exclusion.get("protectionRules", []),
                    "assess_rules": exclusion.get("assessmentRules", []),
                }
                for exclusion in ts_exclusions.get("urlExceptions", [])
            ],
            "application.input_exclusions": [
                {
                    "name": exclusion.get("name", ""),
                    "modes": exclusion.get("modes", []),
                    "urls": exclusion.get("urls", []),
                    "match_strategy": exclusion.get("matchStrategy", ""),
                    "assess_rules": exclusion.get("assessmentRules", []),
                    "protect_rules": exclusion.get("protectionRules", []),
                    "input_type": exclusion.get("inputType", ""),
                    "input_name": exclusion.get("inputName", ""),
                }
                for exclusion in ts_exclusions.get("inputExceptions", [])
            ],
        }
